==> retro_data_structures-0.28.0-py3-none-any.whl: retro_data_structures/properties/echoes/archetypes/TweakGui_ScannableObjectDownloadTimes.py <==
# Generated File
from __future__ import annotations

import dataclasses
import struct
import typing
import typing_extensions

from retro_data_structures import json_util
from retro_data_structures.game_check import Game
from retro_data_structures.properties.base_property import BaseProperty
from retro_data_structures.properties.field_reflection import FieldReflection

if typing.TYPE_CHECKING:
    class TweakGui_ScannableObjectDownloadTimesJson(typing_extensions.TypedDict):
        fast: float
        slow: float
    

_FAST_FORMAT = None
_FAST_IDS = (0xea8b1d10, 0xb1338beb)


@dataclasses.dataclass()
class TweakGui_ScannableObjectDownloadTimes(BaseProperty):
    fast: float = dataclasses.field(default=1.0, metadata={
        'reflection': FieldReflection[float](
            float, id=0xea8b1d10, original_name='Fast'
        ),
    })
    slow: float = dataclasses.field(default=3.0, metadata={
        'reflection': FieldReflection[float](
            float, id=0xb1338beb, original_name='Slow'
        ),
    })

    @classmethod
    def game(cls) -> Game:
        return Game.ECHOES

    @classmethod
    def from_stream(cls, data: typing.BinaryIO, size: int | None = None, default_override: dict | None = None) -> typing_extensions.Self:
        property_count = struct.unpack(">H", data.read(2))[0]
        if (result := cls._fast_decode(data, property_count)) is not None:
            return result

        present_fields = default_override or {}
        for _ in range(property_count):
            property_id, property_size = struct.unpack(">LH", data.read(6))
            start = data.tell()
            try:
                property_name, decoder = _property_decoder[property_id]
                present_fields[property_name] = decoder(data, property_size)
            except KeyError:
                raise RuntimeError(f"Unknown property: 0x{property_id:08x}")
            assert data.tell() - start == property_size

        return cls(**present_fields)

    @classmethod
    def _fast_decode(cls, data: typing.BinaryIO, property_count: int) -> typing_extensions.Self | None:
        if property_count != 2:
            return None
    
        global _FAST_FORMAT
        if _FAST_FORMAT is None:
            _FAST_FORMAT = struct.Struct('>LHfLHf')
    
        dec = _FAST_FORMAT.unpack(data.read(20))
        assert (dec[0], dec[3]) == _FAST_IDS
        return cls(
            dec[2],
            dec[5],
        )

    def to_stream(self, data: typing.BinaryIO, default_override: dict | None = None) -> None:
        default_override = default_override or {}
        data.write(b'\x00\x02')  # 2 properties

        data.write(b'\xea\x8b\x1d\x10')  # 0xea8b1d10
        data.write(b'\x00\x04')  # size
        data.write(struct.pack('>f', self.fast))

        data.write(b'\xb13\x8b\xeb')  # 0xb1338beb
        data.write(b'\x00\x04')  # size
        data.write(struct.pack('>f', self.slow))

    @classmethod
    def from_json(cls, data: json_util.JsonValue) -> typing_extensions.Self:
        json_data = typing.cast("TweakGui_ScannableObjectDownloadTimesJson", data)
        return cls(
            fast=json_data['fast'],
            slow=json_data['slow'],
        )

    def to_json(self) -> json_util.JsonObject:
        return {
            'fast': self.fast,
            'slow': self.slow,
        }

    def dependencies_for(self, asset_manager):
        yield from []


def _decode_fast(data: typing.BinaryIO, property_size: int):
    return struct.unpack('>f', data.read(4))[0]


def _decode_slow(data: typing.BinaryIO, property_size: int):
    return struct.unpack('>f', data.read(4))[0]


_property_decoder: typing.Dict[int, typing.Tuple[str, typing.Callable[[typing.BinaryIO, int], typing.Any]]] = {
    0xea8b1d10: ('fast', _decode_fast),
    0xb1338beb: ('slow', _decode_slow),
}

==> retro_data_structures-0.29.0-py3-none-any.whl: retro_data_structures/properties/echoes/archetypes/TweakGui_ScannableObjectDownloadTimes.py <==
# Generated File
from __future__ import annotations

import dataclasses
import struct
import typing
import typing_extensions

from retro_data_structures import json_util
from retro_data_structures.game_check import Game
from retro_data_structures.properties.base_property import BaseProperty
from retro_data_structures.properties.field_reflection import FieldReflection

if typing.TYPE_CHECKING:
    from retro_data_structures.asset_manager import AssetManager
    from retro_data_structures.base_resource import Dependency

    class TweakGui_ScannableObjectDownloadTimesJson(typing_extensions.TypedDict):
        fast: float
        slow: float
    

_FAST_FORMAT: struct.Struct | None = None
_FAST_IDS = (0xea8b1d10, 0xb1338beb)


@dataclasses.dataclass()
class TweakGui_ScannableObjectDownloadTimes(BaseProperty):
    fast: float = dataclasses.field(default=1.0, metadata={
        'reflection': FieldReflection[float](
            float, id=0xea8b1d10, original_name='Fast'
        ),
    })
    slow: float = dataclasses.field(default=3.0, metadata={
        'reflection': FieldReflection[float](
            float, id=0xb1338beb, original_name='Slow'
        ),
    })

    @classmethod
    def game(cls) -> Game:
        return Game.ECHOES

    @classmethod
    def from_stream(cls, data: typing.BinaryIO, size: int | None = None, default_override: dict | None = None) -> typing_extensions.Self:
        property_count = struct.unpack(">H", data.read(2))[0]
        if (result := cls._fast_decode(data, property_count)) is not None:
            return result

        present_fields = default_override or {}
        for _ in range(property_count):
            property_id, property_size = struct.unpack(">LH", data.read(6))
            start = data.tell()
            try:
                property_name, decoder = _property_decoder[property_id]
                present_fields[property_name] = decoder(data, property_size)
            except KeyError:
                raise RuntimeError(f"Unknown property: 0x{property_id:08x}")
            assert data.tell() - start == property_size

        return cls(**present_fields)

    @classmethod
    def _fast_decode(cls, data: typing.BinaryIO, property_count: int) -> typing_extensions.Self | None:
        if property_count != 2:
            return None
    
        global _FAST_FORMAT
        if _FAST_FORMAT is None:
            _FAST_FORMAT = struct.Struct('>LHfLHf')
    
        dec = _FAST_FORMAT.unpack(data.read(20))
        assert (dec[0], dec[3]) == _FAST_IDS
        return cls(
            dec[2],
            dec[5],
        )

    def to_stream(self, data: typing.BinaryIO, default_override: dict | None = None) -> None:
        default_override = default_override or {}
        data.write(b'\x00\x02')  # 2 properties

        data.write(b'\xea\x8b\x1d\x10')  # 0xea8b1d10
        data.write(b'\x00\x04')  # size
        data.write(struct.pack('>f', self.fast))

        data.write(b'\xb13\x8b\xeb')  # 0xb1338beb
        data.write(b'\x00\x04')  # size
        data.write(struct.pack('>f', self.slow))

    @classmethod
    def from_json(cls, data: json_util.JsonValue) -> typing_extensions.Self:
        json_data = typing.cast("TweakGui_ScannableObjectDownloadTimesJson", data)
        return cls(
            fast=json_data['fast'],
            slow=json_data['slow'],
        )

    def to_json(self) -> json_util.JsonObject:
        return {
            'fast': self.fast,
            'slow': self.slow,
        }

    def dependencies_for(self, asset_manager: AssetManager) -> typing.Iterator[Dependency]:
        yield from []


def _decode_fast(data: typing.BinaryIO, property_size: int) -> float:
    return struct.unpack('>f', data.read(4))[0]


def _decode_slow(data: typing.BinaryIO, property_size: int) -> float:
    return struct.unpack('>f', data.read(4))[0]


_property_decoder: dict[int, tuple[str, typing.Callable[[typing.BinaryIO, int], typing.Any]]] = {
    0xea8b1d10: ('fast', _decode_fast),
    0xb1338beb: ('slow', _decode_slow),
}
